@@ -1,5 +1,5 @@
 /* [ldsh.c wk 03.11.93] a linker frontend
- *	Copyright (c) 1993 by Werner Koch (dd9jn)
+ *	Copyright (c) 1993,1997 by Werner Koch (dd9jn)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,16 +14,22 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *
+ * BUGS: if else endif kann durch einen Kommentar in einem else.zweig
+ *	 durcheinandergebracht werden.
  */
 
 #include <wk/tailor.h>
-RCSID("$Id: ldsh.c,v 1.18 1996/08/30 17:23:08 wk Exp $")
+RCSID("$Id: ldsh.c,v 1.25 1997/08/25 09:14:45 wk Exp $")
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
 #include <wk/string.h>
+#include <wk/direc.h>
 #include <wk/file.h>
+#include <wk/io.h>
 
 
 #ifndef UNIX
@@ -102,30 +108,29 @@ static struct {
 	int syslink;
 	int nolinkinfo;
 	char *configfile;
+	const char *toolprefix;
     } opt ;
 
 static struct {
     const char *s;
     mtype_t type;
+    int flag;	/* bit 0:  unix, bit 1: dos etc.  bit 2: cross unix to dos */
 } extTable[] = {
-  #ifdef UNIX
-    { ".o"  , mOBJ },
-    { ".a"  , mLIB },
-    { ".sa" , mLIB },
-    { ".so" , mEXE },
-    { ""    , mEXE },
-  #else
-    { ".obj"  , mOBJ },
-    { ".lib"  , mLIB },
-    { ".imp"  , mLIB }, /* translated to mIMP for netware */
-    { ".exp"  , mEXP },
-    { ".exe"  , mEXE },
-  #endif
-    { ".dll"  , mEXE },
-    { ".nlm"  , mEXE },
-    { ".def"  , mDEF },
-    { ".map"  , mMAP },
-    { ".res"  , mRES },
+    { ".o"    , mOBJ, 1 },
+    { ".a"    , mLIB, 1 },
+    { ".sa"   , mLIB, 1 },
+    { ".so"   , mEXE, 1 },
+    { ".obj"  , mOBJ, 2 },
+    { ".lib"  , mLIB, 2 },
+    { ".imp"  , mLIB, 2 }, /* translated to mIMP for netware */
+    { ".exp"  , mEXP, 2 },
+    { ".exe"  , mEXE, 2 },
+    { ".dll"  , mEXE, 0 },
+    { ".nlm"  , mEXE, 0 },
+    { ".def"  , mDEF, 0 },
+    { ".map"  , mMAP, 0 },
+    { ".res"  , mRES, 0 },
+    { ""      , mEXE, 1 },
     { NULL, 0 }
 } ;
 
@@ -149,6 +154,7 @@ static MACRO macrolist;
 static char *resourceCompiler = "rc";
 
 static linker_t  linker;
+static const char *linker_command;
 static filelist_t *fileTable;
 static int linkerReturnCode;
 
@@ -158,16 +164,20 @@ static char  *infoStringArea;
 static size_t infoStringAreaUsed;
 static size_t infoStringAreaSize;
 
+static int import_lib_ready;
 
 /****** prototypes ********/
 static void Cleanup( void *dummy );
 static char *ChangeSlash( char *string );
 static void FindLinker(void);
 static void SetupFiletype(void);
+#ifdef UNIX
+  static void CallDLLTool(void);
+#endif
 static void CallLinker(void);
 static void CallResourceLinker(void);
 static void CallImportLinker(void);
-static void RunUNIXLinker(void);
+static void RunUNIXLinker(const char *);
 static void BuildMSLink( FILE *fp );
 static void BuildWatcomLink( FILE *fp );
 static void WatcomExtractFromDef( FILE *fpout, FILE *fpin, const char *key );
@@ -187,38 +197,16 @@ CopyRight( int level )
     const char *p;
     switch( level ) {
       case 10:
-      case 0:	p = "ldsh - v1.00c; "
-		    "Copyright 1993-96 by Werner Koch (dd9jn)" ; break;
-      case 14:	p =	    "1.00c"; break;
+      case 0:	p = "ldsh - v1.10b; "
+		    "Copyright 1993-97 by Werner Koch (dd9jn)" ; break;
+      case 14:	p =	    "1.10b"; break;
       case 1:
       case 11:	p = "Usage: ldsh [options] modules (-h for help)";
 		break;
       case 2:
       case 12:	p =
-    "\nSyntax: ldsh [options] modules\n"
-    "LDSH calls a linker depending on the Compiler and the OS\n"
-    "Options summary:\n"
-    " -b[<machine>] force the use of gcc and optional with parameter -b\n"
-    " -s<n> = set stacksize to n bytes\n"
-    " -g = enable debugging\n"
-    " -pg = enable profiling\n"
-    " -w = enable windowapi\n"
-    " -i = retain an incomplete run file\n"
-    " -m = force creation of a map file\n"
-    " -n = no execution\n"
-    " -jn= no default library search\n"
-    " -ji= be case insensitive\n"
-    " -jx= do not create a linker-info object\n"
-    " -L<name> = create import library <name> for the DLL\n"
-    " -D<k>=<s> = insert string 's' with key 'k' into the executable\n"
-    " -Wl<string> = pass string to the linker\n"
-  #ifdef UNIX
-    " -posix = use posix linker (for HP)\n"
-  #else
-    " -syslink = force usage of system linker\n",
-  #endif
-    " -C<file> = use this configurationfile\n"
-    " -h = help\n";
+    "Syntax: ldsh [options] modules\n"
+    "LDSH calls a linker depending on the Compiler and the OS\n";
 	break;
       case 13:	p = "ldsh"; break;
       default:	p = WklibCopyRight(level);
@@ -227,100 +215,81 @@ CopyRight( int level )
     return p;
 }
 
+static ARGPARSE_OPTS main_opts[] = {
+    { 'v', "verbose",   0   , "Tell what we are doing" },
+    { 'd', "debug",     0   , "Increase debug level" },
+    { 'm', "force-map" ,0   , "force creation of a map file" },
+    { 'g', NULL        ,0   , "enable debugging" },
+    { 'w', "window-api",1   , "enable windowapi" },
+    { 'i', "ignore"  ,  2   , "retain an incomplete run file" },
+    { 's', "stacksize", 3   , "set stacksize to n bytes" },
+    { 'n', "dry-run",   0   , "do not execute commands" },
+    { 'L', NULL,        2   , "create import library <name> for the DLL" },
+    { 'C', "config",    2   , "use this configurationfile" },
+    { 'b', "machine",   2|8 , "force the use of gcc (optional with arg -b)" },
+    { 'B', "exec-prefix", 2 , "prefix used to call special tools" },
+    { 'D', NULL,        2   , "insert string <s> with key <k> into the\n"
+			      "executable (e.g. -D name=joe" },
+    { 501, "jn",        0   , "no default library search" },
+    { 502, "ji",        0   , "be case in-sensitive" },
+    { 503, "jx",        0   , "do not create a linker-info object" },
+    { 601, "syslink" ,  0   , "force usage of system linker" },
+    { 602, "Wl"      ,  2   , "pass string to the linker" },
+    { 603, "posix"   ,  0   , "use posix linker (for HP)" },
+    { 604, "pg"      ,  0   , "enable profiling" },
+    {0} };
+
+
 void
 main( int argc, char **argv )
 {
     char *s;
     filelist_t *f, *fhelp;
     int others;
+    ARGPARSE_ARGS pargs;
 
     if( ArgExpand( &argc, &argv, 4 | 1 ) )
 	Error(4,GetStr(12));
 
-    for( s=""; --argc && **++argv == '-' && *s != '-'; )
-	for( s = *argv + 1 ; *s && *s != '-' ; s++ )
-	    switch( *s ) {
-	     case '#' : opt.debug++; break;
-	     case 'v' : opt.verbose++; break;
-	     case 's' :
-		if( !strcmp(s, "syslink") ) {
-		    opt.syslink++;
-		    while( *s ) s++; s--;
-		}
-		else {
-		    if( !isdigit(s[1]) )
-			CopyRight(1) ;
-		    opt.stackSize = atol( ++s ) ;
-		    opt.stackSizeForced++;
-		    while( isdigit(*s) ) s++ ; s-- ;
-		}
-		break ;
-	      case 'm' : opt.forcemap++; break;
-	      case 'n' : opt.noExec++; break;
-	      case 'g' : opt.setDebug++; break;
-	      case 'w' : opt.windowapi++; break;
-	      case 'i' : opt.ignore++; break;
-	      case 'j' :
-		switch( s[1] ) {
-		  case 'n' : opt.nodefault++; break;
-		  case 'i' : opt.nocase++; break;
-		  case 'x' : opt.nolinkinfo++; break;
-		  default: goto invalidOption;
-		}
-		s++;
-		break;
-	      case 'W' :
-		if( s[1] == 'l' ) {
-		    opt.linkerOptions = s+2;
-		    while( *s )
-			s++;
-		    s--;
-		}
-		else
-		    goto invalidOption;
-		break;
+    pargs.argc = &argc;
+    pargs.argv = &argv;
+    pargs.flags= 32; /* enable kludge to allow single dashed long options */
+    while( ArgParse( &pargs, main_opts) ) {
+	s = pargs.r.ret_str;
+	switch( pargs.r_opt ) {
+	  case 'v': opt.verbose++; break;
+	  case 'd': opt.debug++; break;
+	  case 'm': opt.forcemap++; break;
+	  case 'n': opt.noExec++; break;
+	  case 'g': opt.setDebug++; break;
+	  case 'w': opt.windowapi++; break;
+	  case 'i': opt.ignore++; break;
+	  case 's':
+	    opt.stackSizeForced++;
+	    opt.stackSize = pargs.r.ret_long;
+	    break;
+	  case 'L': opt.implibName = pargs.r.ret_str; break;
+	  case 'C': opt.configfile = pargs.r.ret_str; break;
+	  case 'b': opt.machine = pargs.r_type? pargs.r.ret_str : ""; break;
+	  case 'B': opt.toolprefix = pargs.r.ret_str; break;
+	  case 'D': AddInfoString(pargs.r.ret_str); break;
 
-	      case 'L' :
-		opt.implibName = s+1;
-		while( *s )
-		    s++;
-		s--;
+	  case 501: opt.nodefault++; break;
+	  case 502: opt.nocase++; break;
+	  case 503: opt.nolinkinfo++; break;
+	  case 601: opt.syslink++; break;
+	  case 602: opt.linkerOptions = pargs.r.ret_str; break;
+	  case 603: opt.posix++; break;
+	  case 604: opt.profile++; break;
+	  case -2 : /* invalid option */
+	    if( pargs.r.ret_str && !strncmp(pargs.r.ret_str,"Wl",2) ) {
+		/* Oh what a kludge to make thinks like "-Wl,-egal" work */
+		opt.linkerOptions = pargs.r.ret_str+2;
 		break;
-	      case 'C' :
-		opt.configfile = s+1;
-		while( *s )
-		    s++;
-		s--;
-		break;
-	      case 'b' :
-		opt.machine = s+1;
-		while( *s )
-		    s++;
-		s--;
-		break;
-	      case 'D' :
-		AddInfoString(s+1);
-		while( *s )
-		    s++;
-		s--;
-		break;
-	      case 'p' :
-		if( !strcmp(s, "posix") )
-		    opt.posix++;
-		else if( !strcmp(s, "pg") )
-		    opt.profile++;
-		else
-		    goto invalidOption;
-		while( *s )
-		    s++;
-		s--;
-		break;
-	      case 'h' :
-	      case '?' : CopyRight(0) ; CopyRight(2); break;
-
-	      invalidOption:
-	      default  : Error(3,GetStr(15),s );
 	    }
+	  default : pargs.err = 2; break;
+	}
+    }
     if( !argc )
 	CopyRight(1);
 
@@ -370,14 +339,20 @@ main( int argc, char **argv )
     target_CPU = xstrdup(target);
     if( s = strchr(target_CPU,'-') ) {
 	*s++ = 0;
-	target_COMPANY = s;
-	if( s = strchr(target_COMPANY,'-') ) {
-	    *s++ = 0;
-	    target_SYSTEM = s;
+	if( *s == '-' ) {
+	    target_COMPANY = "";
+	    target_SYSTEM = s+1;
+	}
+	else {
+	    target_COMPANY = s;
+	    if( s = strchr(target_COMPANY,'-') ) {
+		*s++ = 0;
+		target_SYSTEM = s;
+	    }
 	}
     }
     if( !target_CPU || !*target_CPU
-	|| !target_COMPANY || !*target_COMPANY
+	|| !target_COMPANY
 	|| !target_SYSTEM || !*target_SYSTEM )
 	Error(2,"malformed TARGET=%s", target);
 
@@ -389,10 +364,16 @@ main( int argc, char **argv )
 
     FindLinker();
     SetupFiletype();
+  #ifdef UNIX
+    if( createDLL && (	  !strcmp( target_SYSTEM, "mingw32")
+		       || !strcmp( target_SYSTEM, "cygwin32")
+		       || !strcmp( target_SYSTEM, "winnt") ) )
+	CallDLLTool();
+  #endif
     CallLinker();
     if( !linkerReturnCode )
 	CallResourceLinker();
-    if( opt.implibName && !linkerReturnCode )
+    if( opt.implibName && !linkerReturnCode && !import_lib_ready )
 	CallImportLinker();
     if( linkerReturnCode > 255 )
 	linkerReturnCode = 4;
@@ -435,7 +416,8 @@ Cleanup( void *dummy )
 
 
 
-static char *ChangeSlash( char *string )
+static char *
+ChangeSlash( char *string )
 {
     char *s;
 
@@ -451,18 +433,17 @@ static char *ChangeSlash( char *string )
  * Bestimmt den Typ des Linkers
  */
 
-static void FindLinker()
+static void
+FindLinker()
 {
     char *os;
 
     ostype = oOS2;
     isOS2Version2 = 1;
-  #if UNIX
-    os= "unix";
-  #else
     os = target_SYSTEM;
-  #endif
 
+    if( opt.verbose )
+	Info("os='%s' cpu='%s'", os, target_CPU );
     if( !strcmp( os, "os2") && !strcmp(target_CPU,"i386") ) {
 	linker = tLINK386;
     }
@@ -478,6 +459,16 @@ static void FindLinker()
 	linker = tLD;
 	ostype = oUNIX;
     }
+    else if( !strcmp( os, "mingw32") && !strcmp(target_CPU,"i386") ) {
+	if( opt.verbose )
+	    linker_command="gcc -v";
+	else
+	    linker_command="gcc";
+	if( !opt.toolprefix )
+	    opt.toolprefix="i386--mingw32";
+	linker = tLD;
+	ostype = oUNIX;
+    }
     else if( !strcmp( os, "msdos" ) && !strcmp(target_CPU,"i386")) {
 	linker = tLINKdos;
 	resourceCompiler = "rc_d";
@@ -496,6 +487,7 @@ static void FindLinker()
     }
     else {  /* assume a UNIX system */
 	linker = tLD;
+	ostype = oUNIX;
     }
 
     if( opt.syslink )
@@ -627,11 +619,156 @@ SetupFiletype()
 	}
 }
 
+
+/****************
+ * This is used with mingw32 (or cygwin32 ) to create the dll export object
+ * and the import library (when used with option -L)
+ *
+ * NOTE: We need "dlltool"  somewhere in the path
+ *
+ * Example: to create the dll "foo.dll" we use this command:
+ *   dlltool --output-exp temp.o --output-lib implibname.a
+ *	     --as i386--mingw32-as --def foo.def --dllname foo.dll foo.o
+ * "--output-lib" is only used with option -L.
+ * "--def" is only used if we have a def file
+ * "foo" is the first object file encountered and only used if there is no .def.
+ */
+#ifdef UNIX
+static void
+CallDLLTool()
+{
+    filelist_t *f;
+    const char *expFile=NULL, *objFile=NULL, *defFile=NULL;
+    char *nameFile=NULL, *baseFile=NULL;
+    char *s, *p;
+    t_fplist fpl;
+
+    /* Get the def file */
+    for(f=fileTable; f; f = f->next )
+	if( f->type == mDEF ) {
+	    defFile = f->name;
+	    break;
+	}
+    if( !defFile ) { /* no def file, so take the first object file */
+	for(f=fileTable; f; f = f->next )
+	    if( f->type == mOBJ )
+		objFile = f->name;
+	if( !objFile )
+	    Error(2,"error: need .def or .o file");
+    }
+    for(f=fileTable; f; f = f->next )
+	if( f->type == mEXE ) {
+	    nameFile = basename(f->name);
+	    break;
+	}
+    if( !nameFile )
+	Error(2,"error: need .dll file to get the name of the DLL");
+
+    /* We cannot use this function because dlltools is very lazy, by
+     * creating a tempfile by prefixing this filename with a "t" ---
+     * What we would get ist this "t/tmp/..." and that always, because
+     * the CreateTmpFile() functions return absolute filenames ....
+     *
+     * if( !(p = CreateTmpFile3( "", TMP_PREFIX, ".exp" )) )
+     *	 Error(2,GetStr(8));
+     *
+     * So we a more simple scheme.
+     */
+    p = xmalloc(50);
+    sprintf(p, "exp-%d.tmp", (int)getpid() );
+    fpl = xcalloc( 1, sizeof *fpl );
+    fpl->name = p;
+    fpl->next = tempfiles;
+    tempfiles = fpl;
+    expFile = p;
+    /* and insert the exp file into the list of object files */
+    f = xcalloc( 1, sizeof *f + strlen(expFile) );
+    strcpy( f->name, expFile );
+    ChangeSlash( f->name );
+    f->type =mOBJ;
+    f->next = fileTable;
+    fileTable = f;
+
+    if( !(baseFile = CreateTmpFile3( NULL, TMP_PREFIX, ".base" )) )
+	Error(2,GetStr(8));
+    fpl = xcalloc( 1, sizeof *fpl );
+    fpl->name = baseFile;
+    fpl->next = tempfiles;
+    tempfiles = fpl;
+
+    s = xmalloc( 120
+		 + strlen(expFile)
+		 + strlen(defFile? defFile : objFile)
+		 + strlen(nameFile)
+		 + strlen(baseFile)
+		 + (opt.implibName? strlen(opt.implibName):0)
+	       );
+    sprintf(s,"dlltool %s--as %s%sas --output-exp %s %s%s "
+	      "--dllname %s %s%s %s",
+	       opt.verbose? "-v ":"",
+	       opt.toolprefix? opt.toolprefix : "",
+	       opt.toolprefix? "-"            : "",
+	       expFile,
+	       opt.implibName? "--output-lib ":"",
+	       opt.implibName? opt.implibName :"",
+	       nameFile,
+	       defFile? "--def ":"",
+	       defFile? defFile :"",
+	       objFile? objFile :"" );
+
+    if( opt.noExec )
+	printf("Command: '%s'\n", s );
+    else {
+	Info("(%.100s)", s );
+	linkerReturnCode = system(s);
+	if( opt.debug > 1)
+	    Info("dbg: system(%.100s) returned %d", s, linkerReturnCode );
+	if( linkerReturnCode )
+	    Error(2,"first dlltool call failed (rc=%d)", linkerReturnCode );
+    }
+
+    /* now run the unix linker to produce the base relocation table */
+    sprintf(s,"-Wl,--base-file -Wl,%s", baseFile );
+    RunUNIXLinker(s);
+    if( linkerReturnCode )
+	Error(2,"first link failed (rc=%d)", linkerReturnCode );
+
+    sprintf(s,"dlltool %s--as %s%sas --output-exp %s %s%s "
+	      "--dllname %s --base-file %s %s%s %s",
+	       opt.verbose? "-v ":"",
+	       opt.toolprefix? opt.toolprefix : "",
+	       opt.toolprefix? "-"            : "",
+	       expFile,
+	       opt.implibName? "--output-lib ":"",
+	       opt.implibName? opt.implibName :"",
+	       nameFile,
+	       baseFile,
+	       defFile? "--def ":"",
+	       defFile? defFile :"",
+	       objFile? objFile :"" );
+
+    if( opt.noExec )
+	printf("Command: '%s'\n", s );
+    else {
+	Info("(%.100s)", s );
+	linkerReturnCode = system(s);
+	if( opt.debug > 1)
+	    Info("dbg: system(%.100s) returned %d", s, linkerReturnCode );
+	if( linkerReturnCode )
+	    Error(2,"second dlltool call failed (rc=%d)", linkerReturnCode );
+    }
+
+    free(s);
+    free(nameFile);
+}
+#endif /* UNIX */
+
 /****************
  * Create a responsefile and call the linker
  */
 
-static void CallLinker()
+static void
+CallLinker()
 {
     FILE *fp;
     char *filename, *s;
@@ -711,7 +848,7 @@ static void CallLinker()
 	Bug("The Linker for this System is not supported");
 
     if( linker == tLD )
-	RunUNIXLinker();
+	RunUNIXLinker(NULL);
     else if( opt.noExec )
 	printf("Command: '%s'\n", s );
     else {
@@ -854,7 +991,7 @@ CallImportLinker()
  */
 
 static void
-RunUNIXLinker()
+RunUNIXLinker( const char *more_opts )
 {
     filelist_t *f;
     size_t n;
@@ -864,24 +1001,44 @@ RunUNIXLinker()
     for(n=0,f=fileTable; f; f = f->next )
 	n += strlen(f->name) + 1;
     /* add something for the options */
-    n += 100;
+    n += 146;
     if( opt.machine )
 	n += strlen(opt.machine) + 4;
     if( opt.linkerOptions )
 	n += strlen(opt.linkerOptions);
+    if( linker_command )
+	n += strlen( linker_command );
+    if( opt.toolprefix )
+	n += strlen( opt.toolprefix );
+    if( more_opts )
+	n += strlen( more_opts );
     commandline = xmalloc(n);
     if( opt.machine ) {
 	p = stpcpy(commandline,"gcc ");
 	if( *opt.machine )
 	    p = stpcpy(stpcpy(stpcpy(p,"-b "),opt.machine)," ");
     }
+    else if( linker_command && opt.toolprefix )
+	p = stpcpy(stpcpy(stpcpy(stpcpy(
+		   commandline,opt.toolprefix),"-"),linker_command)," ");
+    else if( linker_command )
+	p = stpcpy(stpcpy(commandline,linker_command)," ");
     else
 	p = stpcpy(commandline,opt.posix ? "c89 " : "cc ");
     if( opt.profile )
 	p = stpcpy(p, "-pg ");
+    if( createDLL && !strcmp(target_SYSTEM,"mingw32") )
+	p = stpcpy(p,"-dll ");
+    if( opt.windowapi && !strcmp(target_SYSTEM,"mingw32") )
+	p = stpcpy(p,"-windows ");
+
     if( opt.linkerOptions )
 	p = stpcpy(stpcpy(stpcpy(p,"-Wl"),opt.linkerOptions)," ");
 
+    if( more_opts )
+	p = stpcpy(stpcpy(p,more_opts)," ");
+
+
     p = stpcpy(p,"-o ");
     /* get the run file */
     for(f=fileTable; f; f = f->next )
@@ -918,9 +1075,10 @@ RunUNIXLinker()
     if( opt.noExec )
 	printf("Command: '%s'\n", commandline );
     else {
+	Info("(%.500s)", commandline );
 	linkerReturnCode = system(commandline);
 	if( opt.debug > 1)
-	    Info("dbg: system(%.100s) returned %d",
+	    Info("dbg: system(%.400s) returned %d",
 				    commandline, linkerReturnCode );
     }
     free(commandline);
@@ -1013,8 +1171,10 @@ static void BuildWatcomLink( FILE *fp )
 	fprintf(fp,"system windows%s\n", createDLL? "_dll":"" );
     else if( ostype == oOS2 && !isOS2Version2 )
 	fprintf(fp,"system os2%s\n", createDLL? " dll initinstance":""   );
+    else if( ostype == oOS2 && opt.windowapi && createDLL )
+	fprintf(fp,"system os2v2 dll initinstance\n"  );
     else if( ostype == oOS2 && opt.windowapi )
-	fprintf(fp,"system os2v2_pm%s\n", createDLL? " dll initinstance":""  );
+	fprintf(fp,"system os2v2_pm\n" );
     else
 	fprintf(fp,"system os2v2%s\n", createDLL? " dll initinstance":""  );
     if( opt.nodefault )
@@ -1294,7 +1454,7 @@ static void WatcomExtractOneFromDef( FILE *fpout, FILE *fpin, const char *key )
  * irgendwanneinmal auf normalen Shell-Syntax umgestellt, z.Z. reicht
  * dieses Verfahren aber aus und es ist auch nicht ntigt, Strings zu erkennen
  * um in diesen keine Expansion vorzunehmen -- in Strings kommen z.Z. nur
- * Ident anwesiungen vor und diese entahlten kein "$(".
+ * Ident Anweisungen vor und diese entahlten kein "$(".
  * Die Lnge eines Makronamens ist z.Z. auf 255 Zeichen limitiert.
  *
  * Verschachtelungen mit Else sind z.Z. noch nicht impl.
@@ -1313,7 +1473,9 @@ static filelist_t *PreprocessFile( filelist_t *f )
 	sCOPY,	    /* copy characters to the outstream */
 	sCOPYNEXT,  /* set state sCOPY for the next character */
 	sSKIP,	    /* do not copy */
-	sPERCENT    /* PERCENT seen in first column */
+	sPERCENT,   /* PERCENT seen in first column */
+	sCONDLINE_NO,  /* This is a line of type "%?name " */
+	sCONDLINE_YES  /* This is a line of type "%?name " */
     } state;
     enum {
       cUNKNOWN = 0,
@@ -1386,6 +1548,14 @@ static filelist_t *PreprocessFile( filelist_t *f )
 	    lfseen = c == '\n';
 	    break;
 
+	  case sCONDLINE_YES:
+	  case sCONDLINE_NO:
+	    if( c == '\n' ) {
+		lfseen = 1;
+		state = sCOPY;
+	    }
+	    break;
+
 	  case sPERCENT:
 	    if( c == '\n' ) {
 		lfseen = 1;
@@ -1463,7 +1633,17 @@ static filelist_t *PreprocessFile( filelist_t *f )
 		    if( isspace(c) ) {
 			buf[idx] = 0;
 			idx = 0;    /* used later on */
-			if( !strcmp( buf, "if" ) )
+			if( *buf == '?' && buf[1] == '!' && buf[2] ) {
+			    cond = cUNKNOWN;
+			    state= !strcmp( buf+2, target_SYSTEM ) ?
+				      sCONDLINE_NO : sCONDLINE_YES;
+			}
+			else if( *buf == '?' && buf[1] ) {
+			    cond = cUNKNOWN;
+			    state= !strcmp( buf+1, target_SYSTEM ) ?
+				      sCONDLINE_YES : sCONDLINE_NO;
+			}
+			else if( !strcmp( buf, "if" ) )
 			    cond = cIN_IF;
 			else if( !strcmp( buf, "elif" ) )
 			    cond = cIN_ELIF;
@@ -1496,6 +1676,7 @@ static filelist_t *PreprocessFile( filelist_t *f )
 			cond = cERROR;
 		    break;
 
+
 		  case cCOMMENT:
 		  case cERROR:
 		    break;
@@ -1516,7 +1697,7 @@ static filelist_t *PreprocessFile( filelist_t *f )
 
 	  default: BUG();
 	}
-	if( state == sCOPY ) {
+	if( state == sCOPY || state == sCONDLINE_YES ) {
 	    if( macroidx != -1 ) {
 		if( macroidx == -2 ) {
 		    if( c == '(' )
@@ -1563,7 +1744,7 @@ static filelist_t *PreprocessFile( filelist_t *f )
     if( fpl )
 	fpl->fp = NULL;
 
-    /* und den named des deffiles ersetzen */
+    /* und den Namen des deffiles ersetzen */
     f2 = xcalloc( 1, sizeof *f2 + strlen(outname) );
     strcpy( f2->name, outname );
     ChangeSlash( f2->name );
