@@ -1050,7 +1050,7 @@ getenv(const char *name)
 }
 
 int
-putenv(const char *string)
+myputenv(const char *string)
 {
     size_t len;
     char **ep, **dp;
@@ -1105,6 +1105,13 @@ putenv(const char *string)
 
     return 0;
 }
+#else
+int 
+myputenv ( const char *str )
+{
+   return putenv (str);
+}
+  
 
 #endif
 
