@@ -184,6 +184,9 @@ struct {
 	{   "enter"     , K_VK_RETURN      },
 	{   "backspace" , K_VK_RUBOUT      },
 	{   "tab"       , K_VK_TAB         },
+	{   "superl"    , K_SUPER_L        },
+	{   "superr"    , K_SUPER_R        },
+	{   "hyperl"    , K_HYPER_R        },
 	{ NULL }
     };
 
@@ -597,10 +600,11 @@ KeyId2String( int keyId, char *buffer, size_t buflen )
  * Returns: ErrorCode
  */
 
-static int ParseCmd( const char *line, cmd_t **retcmd, const char **endp )
+static int
+ParseCmd( const char *line, cmd_t **retcmd, const char **endp )
 {
     int err, i, args, c;
-    size_t len, len1, len2, usedLen;
+    size_t len, len1, len2, len3, usedLen;
     const char *p, *parg, *orgline;
     char *sarg;
     cmd_t cmd;
@@ -655,11 +659,14 @@ static int ParseCmd( const char *line, cmd_t **retcmd, const char **endp )
 	    len1 = strcspn( line, symTerminators );
 	    len2 = len1+strspn( line+len1, whiteSpaces );
 	    len2 += strcspn( line+len2, symTerminators );
+	    len3 = len2+strspn( line+len2, whiteSpaces );
+	    len3 += strcspn( line+len3, symTerminators );
 	    i = (c >= 'a' && c <= 'z') ? cmdLookUp[c-'a'] : 0;
 	    for(; p = command[i].strg ; i++ ) {
 		switch( command[i].flag ) {
 		  case 1 : usedLen = len1; break;
 		  case 2 : usedLen = len2; break;
+		  case 3 : usedLen = len3; break;
 		  default: usedLen = len;  break;
 		}
 		if( command[i].strgLen == usedLen ) {
