@@ -38,6 +38,7 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <signal.h>
+#include <errno.h>
 #endif
 #if __ZTC__
 #include <direct.h>
@@ -141,7 +142,7 @@ int Cmd_Dir( cmd_t *cmd )
  * in den File mit dem Handle fHd das im string angegebene Directory ausgeben
  * dirFlag hat folgende Bewandniss: wenn es beim Aufruf auf 0 steht, so
  * geschieht nicht, ist es beim Aufruf aber auf 1 gesetzt, so wird an String
- * ein / und *.* angehangen, gleich zeitig gibt es folgende Werte zurck:
+ * ein / und *.* angehangen, gleichzeitig gibt es folgende Werte zurck:
  * falls nur ein File gefunden wurde, und dieser war ein Directory, so
  * wird es auf 1 gesetzt, andernfdalls auf 0
  */
@@ -158,6 +159,7 @@ PrintDirectory( int fHd , const char *string, int *dirFlag )
     struct tm *tm;
     long count;
     char convertBuf[F_MAX_FNAME+F_MAX_EXT];
+    int ignore_backups;
   #ifndef UNIX
     enum { pass_DIR, pass_FILES, pass_END } passFlag;
   #endif
@@ -166,6 +168,7 @@ PrintDirectory( int fHd , const char *string, int *dirFlag )
     pathBuffer = xmalloc( F_MAX_PATH );
     lineBuffer = xmalloc( MAX_LINELEN+1);
     xassert( (8+1+8+1+8+1+3+1+F_MAX_PATH ) < MAX_LINELEN );
+    ignore_backups = GetSetOption( SETOPT_IGNORE_BACKUPS );
 
     ddot = 0;
     if( *string ) {
@@ -224,7 +227,9 @@ PrintDirectory( int fHd , const char *string, int *dirFlag )
 	    err = InsertLine( fHd, "01.01.70 00:00:00    <DIR>    ..", 32);
 	while( !err && !ex ) {
 	    p = FindFileStat( &handle, &statBuf );
-	    if( ddot && *p=='.' && p[1]=='.' && !p[2] )
+	    if( ignore_backups && *p && p[strlen(p)-1] == '~' )
+		;
+	    else if( ddot && *p=='.' && p[1]=='.' && !p[2] )
 		;
 	    else if( !(*p == '.' && !p[1]) ) {
 	      #if !UNIX
@@ -292,7 +297,8 @@ const char *GetDirPattern()
  * EIn DOS Command ausfuehren oder die Shell aufrufen
  */
 
-int Cmd_Dos( cmd_t *cmd )
+int
+Cmd_Dos( cmd_t *cmd )
 {
   #if W_FULL
     int err, noprompt;
@@ -330,6 +336,11 @@ int Cmd_Dos( cmd_t *cmd )
 	return 0;
     }
   #endif
+  #if USE_VT_LINUX
+    if( JnxAvailable() )
+	return ERR_USPCMD;
+  #endif
+
     RemoveScreen();
   #if MSDOS && __ZTC__
     swapOpt = GetSetOption( SETOPT_MEMSWAP );
@@ -477,6 +488,8 @@ int Cmd_Detach( const char *string )
 	fclose(st2); st2 = NULL;
 	if( procInfo->autoCmd && (p=strchr( procInfo->cmdline, ';')) )
 	    *p = 0; /* remove autoexec commands */
+	/* Because there is no execvep() we have to do this hack */
+	environ = GetEnvironmentArray();
 	if( !use_shell && !strpbrk( procInfo->cmdline, "*?[]~" ) ) {
 	    /* we do not need the shell for globbing */
 	    if( p = strpbrk( procInfo->cmdline, " \t\n" ) )
