@@ -42,6 +42,10 @@
 #include <ctype.h>
 #include <setjmp.h>
 #include <time.h>
+#include <errno.h>
+#if UNIX
+#include <sys/stat.h>
+#endif
 #include <unistd.h>
 #if OS2 || OS20
   #define INCL_DOSPROCESS 1
@@ -180,6 +184,7 @@ typedef struct {
 	struct stat statbuf;
 	int statbuf_ok; 	/* statbuf is valid */
       #endif
+	char *profile_var;
     } file_t;
 
 typedef struct {
@@ -219,6 +224,7 @@ static int sortCompareCounter;
 static jmp_buf sortCompareJmpBuf;
 static enum { dALFA, dNUM, dNAME, dEXT, dSIZE, dDATE, dALFACI } sortMode;
 static int directorySortKludge;
+static int directorySortReverse;
 
 #if USE_EMS
 static struct {
@@ -327,7 +333,56 @@ CleanUp( void *parm )
 	    ReleaseFile(i);
 }
 
+/****************
+ * Look at the last line of the file for a pattern
+ *  "*-*wedit:abc*-*"
+ * where abc is the value we are interested in. If we found
+ * such a value and if we have a variable "ZPrf_abc" then execute
+ * the contents of this variable.
+ * If this fails, check whether this file would be executed by a
+ * command interpreter and return the "ZPrfSh".  This function can
+ * then later try to figure out the command interpreter and set some
+ * highlighting
+ */
+static char *
+GetProfileName( int fhd )
+{
+    const char *line, *s;
+    ushort nbytes;
+    char *buf;
 
+    SeekLine( fhd, ULONG_MAX );
+    line = GetPtr2Line( fhd, &nbytes );
+    if( nbytes < 13 )
+	goto topline_check;
+    s = MemStr( line, "*-*wedit:", nbytes );
+    if( !s )
+	goto topline_check;
+    nbytes -= s - line;
+    nbytes -= 9;
+    line = s + 9;
+    for(s=line; nbytes; s++, nbytes-- ) {
+	if( isspace(*s) )
+	    goto topline_check;
+	if( *s == '*' )
+	    break;
+    }
+    if( s==line || nbytes < 3 || !( s[1] == '-' && s[2] == '*') )
+	return NULL;
+    nbytes = s - line;
+    buf = xmalloc( 5 + nbytes + 1 );
+    strcpy( buf, "ZPrf_" );
+    memcpy( buf+5, line, nbytes );
+    buf[5+nbytes] = 0;
+    return buf;
+
+  topline_check:
+    SeekLine( fhd, 0 );
+    line = GetPtr2Line( fhd, &nbytes );
+    if( nbytes > 4 && (!strncmp( line, "#!/",3) || !strncmp( line, "# !/",4)))
+	return xstrdup( "ZPrfSh" );
+    return NULL;
+}
 
 /****************
  * Eine File allokieren, die internen Strukturen bereitstellen und
@@ -518,6 +573,7 @@ int AllocFile( int *retfhd, const char *name, int modeFlag,
 	file->hilite_crs = NULL;
 	file->lnr = 0;
 	file->pos = 0;
+	file->profile_var = NULL;
 	file->hiliteFlag = 0;
 	*file->hiliteType = 0;
 	file->nattrib = 1;  /* for the syntax highlighting */
@@ -630,6 +686,7 @@ int AllocFile( int *retfhd, const char *name, int modeFlag,
 	    file->chg = 0;
 	    file->ro  = set2ro;
 	    file->lineIdRover = 0;
+	    file->profile_var = GetProfileName( fhd );
 	    SeekLine( fhd,  0 );
 	    PosLine( fhd, 0 );
 	  #if W_FULL
@@ -733,7 +790,7 @@ static size_t ReadLenPrefixed( char *buffer, FILE *st, int *err)
 	    *err = 1; /* invalid header */
 	return 0;     /* terminate */
     }
-  #if !BIG_ENDIAN
+  #if !WKLIB_BIG_ENDIAN
     len = (len << 8) | ((len >> 8) & 0xff);
   #endif
     if( len > MAX_LINELEN ) {
@@ -768,6 +825,18 @@ static size_t ReadBinary( char *buffer, FILE *st, ushort reclen )
 
 
 
+void
+ExecFilesProfile( int fhd )
+{
+    file_t *file = fileTbl + fhd;
+
+    if( file->profile_var )
+	Cmd_ExecuteString( file->profile_var, ARGTYPE_VAR );
+    else
+	Cmd_ExecuteString( "ZPrf_default", ARGTYPE_VAR );
+}
+
+
 /****************
  * rename an active file
  * checks to see if this file already exists
@@ -904,6 +973,7 @@ void ReleaseFile( int fHd )
 	FREE( file->name );
 	FREE( file->cmdline );
 	FREE( file->editBuf );
+	FREE( file->profile_var );
     }
 }
 
@@ -1319,7 +1389,8 @@ void ResetFileFlag( int fHd, int flag )
 }
 
 
-void UnlockFile( int fHd )
+void
+WeditUnlockFile( int fHd )
 {
     file_t *file;
 
@@ -1534,6 +1605,10 @@ MakeFileName( const char *name )
 {
     char *buf, *newbuf;
     size_t len;
+  #if UNIX
+    int nloop=0;
+    struct stat sb;
+  #endif
 
     name += strspn( name, whiteSpaces ); /* skip Blanks */
   #if OS2  /* wg. HPFS sind auch blanks im Namen moeglich */
@@ -1557,6 +1632,40 @@ MakeFileName( const char *name )
 	buf = xmalloc( F_MAX_PATH+1 );
 	mem2str( buf, name, len+1 );
     }
+
+    /* hmmm, readlink() does not follow a link chain
+     * but the man pages says ELOOP is an errocode, so it
+     * should do it -- Il do it myself here
+     */
+    while( !lstat( buf, &sb ) && S_ISLNK(sb.st_mode) ) { /* this is a symlink */
+	int nchars;
+	int size = strlen(buf)+10;
+	char *buf2;
+	if( size < 100 )
+	    size = 100;
+
+	for(;;) {
+	    buf2 = xmalloc( size );
+	    nchars = readlink( buf, buf2, size-1);
+	    if( nchars < size )
+		break;
+	    free(buf2);
+	    size *= 2;
+	}
+	if( nchars > 0 ) { /* found it */
+	    char *p = buf;
+	    buf = buf2;
+	    free(p);
+	    buf[nchars] = 0; /* terminate the string */
+	    if( ++nloop > 20 ) {
+		Error(0,"%s: loop in symlinks", name );
+		break;
+	    }
+	}
+	else
+	    break;
+    }
+
   #else
     buf = xmalloc( F_MAX_PATH+1 );
     mem2str( buf, name, len+1 );
@@ -2279,6 +2388,7 @@ SortLines( int fileHandle, const char *s )
     memset( &o, 0, sizeof o );
     sortMode = dALFA;
     directorySortKludge = 1;
+    directorySortReverse = 0;
     while( *s ) {
 	if( isspace( *s ) )
 	    s++;
@@ -2292,7 +2402,7 @@ SortLines( int fileHandle, const char *s )
 	    for( ; *s && !isspace(*s); s++ ) {
 		switch( *s ) {
 		  case 'R' : o.nosort++; /* fall thru */
-		  case 'r' : o.rev++; break;
+		  case 'r' : o.rev++; directorySortReverse = 1; break;
 		  case 'c' : o.nocase++; break;
 		  case 'd' : o.dir++;
 		    switch( s[1] ) {
@@ -2707,6 +2817,11 @@ SortLinesCompareDir( line_t __handle **a, line_t __handle **b )
     pb = (*b)->d;
     if( la < 31 || lb < 31) /* kann eigentlich nicht sein ... */
 	return 0;
+
+    /* handle ".." entry, which is always the first one */
+    if( pa[30] == '.' && pa[31]== '.' && !pa[32] )
+	return directorySortReverse? 1: -1;
+
     switch( sortMode ) {
       case dSIZE:
 	sa = atol( pa+18 );
