@@ -780,7 +780,7 @@ int ProcessCmd( cmd_t *cmd )
 
     #if W_FULL
       case CMD_PUTENV:
-	if( putenv( cmd->arg.string ) )
+	if( myputenv( cmd->arg.string ) )
 	    err = ERR_PUTENV;
 	break;
      #endif
