@@ -46,6 +46,7 @@
 #elif DOS386
 #elif DOS16RM
 #elif EMX
+#elif WINNT
 #else
     #error Operating system must either be MSDOS, OS2, or EMX
 #endif
@@ -66,6 +67,7 @@
     #endif
     #include <os2.h>
     #define INCL_WK_WSCRDRV_GUI 1
+#elif WINNT && !__CYGWIN32__
 #else
     #include <bios.h>
     #include <dos.h>
@@ -80,10 +82,14 @@
 #define MAX_DOSKEYS  20  /* our own pretype buffer for MSDOS */
 
 /******* types *************/
+#ifdef WINNT
+typedef CHAR_INFO cell_t;
+#else
 typedef struct {
 	char c;
 	char a;
     } cell_t;
+#endif
 
 #if __IBMC__
 typedef volatile cell_t * crtptr_t;
@@ -97,8 +103,12 @@ typedef volatile cell_t _far *crtptr_t;
 #if OS2 || EMX
     static crtptr_t vioBuffer ;
     static int hvio;
+#elif WINNT
+    static HANDLE con_out, con_in;
+    static int vio_changed;
+    SMALL_RECT viorect;
 #endif
-#if !OS2
+#if !OS2 && !WINNT
     static struct {
 	ushort chr,scan;
 	int state;
@@ -138,6 +148,12 @@ static int translateCodes;
 
 static volatile int *breakFlagAddr;
 
+#ifdef WINNT
+  #define VIDEOPAGE ((int)con_out)
+#else
+  #define VIDEOPAGE 0
+#endif
+
 /********** prototypes *********/
 static cell_t GetRndCell();
 
@@ -150,7 +166,8 @@ static cell_t GetRndCell();
  */
 
 
-int ScrDrvOpen()
+int
+ScrDrvOpen()
 {
     int err=0, cols, rows, pages , i ;
   #if OS20
@@ -160,11 +177,11 @@ int ScrDrvOpen()
   #elif OS2
     crtptr_t vBuffer;
     ushort vioBufferSize;
+  #elif WINNT
   #endif
 
     save.videoMode = -1 ;
-
-  #if DEBUG
+  #if DEBUG && !WINNT
     xassert( sizeof (cell_t) == 2 );
   #endif
   #if OS2 || EMX
@@ -179,7 +196,7 @@ int ScrDrvOpen()
     else {  /* _osmode == DOS_MODE */
    #endif
   #endif
-  #if !OS2  /* DOS or EMX continued */
+  #if !OS2  && !WINNT /* DOS or EMX continued */
 	isColor = 1 ; /* default*/
 	save.videoMode = WKBiosGetVideoMode( &cols, &rows );
 	switch( save.videoMode ) {
@@ -198,7 +215,32 @@ int ScrDrvOpen()
    #if EMX
     }
    #endif
-  #endif /* DOS */
+  #elif WINNT
+
+/*  con_out = CreateFile( "CONOUT$", */
+    if( (con_out = GetStdHandle(STD_OUTPUT_HANDLE)) == INVALID_HANDLE_VALUE ) {
+	Error(0,"GetStdHandle failed: rc=%d", GetLastError() );
+	err =-1;
+	goto leave;
+    }
+    if( (con_in = GetStdHandle(STD_INPUT_HANDLE)) == INVALID_HANDLE_VALUE ) {
+	Error(0,"GetStdHandle failed: rc=%d", GetLastError() );
+	err =-1;
+	goto leave;
+    }
+    save.videoMode = -1;
+    isColor = 1; /* FIXME!*/
+    {	CONSOLE_SCREEN_BUFFER_INFO info;
+
+	if( !GetConsoleScreenBufferInfo( con_out, &info ) ) {
+	    Error(0,"GetConsoleScreenBufferInfo failed: rc=%d", GetLastError());
+	    err =-1;
+	    goto leave;
+	}
+	cols = info.dwSize.X;
+	rows = info.dwSize.Y;
+    }
+  #endif /* !winnt */
 
     /**** Setup Screen Parameters ****/
     screen.sx = cols ;
@@ -208,10 +250,16 @@ int ScrDrvOpen()
     screen.y2 = rows -1 ;
     screen.size = screen.sx * screen.sy ;
     if( !(image = calloc( screen.size, sizeof *image )) )
-	{ err = -1; goto retlab ; }
+	{ err = -1; goto leave; }
 
   #if OS20
     vioBuffer = (crtptr_t)WKVideoGetBuffer( NULL );
+  #elif WINNT
+    vio_changed = 0;
+    viorect.Left  = screen.sx;
+    viorect.Right = 0;
+    viorect.Top = screen.sy;
+    viorect.Bottom = 0;
   #elif OS2 || EMX
    #if EMX
     if( _osmode == OS2_MODE ) {
@@ -235,7 +283,7 @@ int ScrDrvOpen()
     save.orgScrRows = screen.sy;
     save.orgScrCols = screen.sx;
     if( !(save.orgScr = malloc( save.orgScrSize * sizeof *save.orgScr)) )
-	{ err = -1; goto retlab ; }
+	{ err = -1; goto leave ; }
     ScrDrvMouseOn();
     ScrDrvSaveScreen();
     ScrDrvShowCrs( 1 ); /* initial painting of cursor */
@@ -243,7 +291,7 @@ int ScrDrvOpen()
     ScrDrvRepaint();	/* Sync with image */
     MouseShow();
 
-  retlab:
+  leave:
     if( err && save.videoMode != -1 )  /* Reset Videomode on error*/
 	WKBiosSetVideoMode( save.videoMode ) ;
     if( err )
@@ -254,13 +302,111 @@ int ScrDrvOpen()
 
 
 
+#if INCL_WK_WSCRDRV_GUI
 HAB ScrDrvGetActiveHAB(void)
 {
     return 0;
 }
+#endif
 
 
+#ifdef WINNT
 
+unsigned
+WKKeyboardGetToggle()
+{
+  #if 0
+    KBDINFO kbi;
+
+    kbi.cb = sizeof kbi;
+    if( !KbdGetStatus( &kbi, 0 ) )
+	return (kbi.fsState >> 4) & 0x07;
+    else
+	return 0;
+  #endif
+	return 0;
+}
+
+
+/****************
+ * Get a character from the keyboard
+ * mode 0 ::= no wait
+ *	1 ::= wait
+ *
+ * Returns: 0 = Character available
+ *	    !0 = no Character or error
+ */
+
+int
+WKKeyboardGetChar(int mode, ushort *chr, ushort *scan, unsigned *state)
+{
+    INPUT_RECORD irec;
+    ulong n;
+
+    if( !mode ) {
+	if( !GetNumberOfConsoleInputEvents(con_in, &n ) ) {
+	    Error(0,"GetNumberOfConsoleInputEvents failed: rc=%d",
+						GetLastError() );
+	    return 1;
+	}
+	if( !n )
+	    return 2; /* no evenets */
+    }
+
+  repeat:
+    if( !ReadConsoleInput(con_in, &irec, 1, &n ) ) {
+	Error(0,"ReadConsoleInput failed: rc=%d", GetLastError() );
+	return 1;
+    }
+
+    switch( irec.EventType ) {
+      case KEY_EVENT:
+	if( !irec.Event.KeyEvent.bKeyDown ) {
+	    if( mode )
+		goto repeat;
+	    return 2;
+	}
+      #if 0
+	Info("vkey=%3hu scan=%3hu char=%3u (%c) rep=%hu state=%lx",
+		irec.Event.KeyEvent.wVirtualKeyCode,
+		irec.Event.KeyEvent.wVirtualScanCode,
+		irec.Event.KeyEvent.uChar.AsciiChar,
+		irec.Event.KeyEvent.uChar.AsciiChar >=' '
+		&& irec.Event.KeyEvent.uChar.AsciiChar <='z' ?
+		irec.Event.KeyEvent.uChar.AsciiChar : '?',
+		irec.Event.KeyEvent.wRepeatCount,
+		irec.Event.KeyEvent.dwControlKeyState  );
+      #endif
+	if( scan )
+	    *scan = irec.Event.KeyEvent.wVirtualScanCode;
+	if( chr ) {
+	    if( irec.Event.KeyEvent.dwControlKeyState & ENHANCED_KEY )
+		*chr = 224;
+	    else
+		*chr  = irec.Event.KeyEvent.uChar.AsciiChar;
+	}
+	if( state ) {
+	    *state  = 0;
+	    if( irec.Event.KeyEvent.dwControlKeyState & SHIFT_PRESSED )
+		*state |= K_FLAG_SHIFT;
+	    if( irec.Event.KeyEvent.dwControlKeyState
+		& (LEFT_CTRL_PRESSED | RIGHT_CTRL_PRESSED) )
+		*state |= K_FLAG_CTRL;
+	    if( irec.Event.KeyEvent.dwControlKeyState & LEFT_ALT_PRESSED )
+		*state |= K_FLAG_ALT;
+	    if( irec.Event.KeyEvent.dwControlKeyState & RIGHT_ALT_PRESSED )
+		*state |= K_FLAG_COMPOSE;
+	}
+	return 0;
+      default:
+	if( mode )
+	    goto repeat; /* we want a char and nothing else */
+	return 2; /* unsupported event */
+    }
+}
+
+
+#endif /* WINNT */
 
 
 
@@ -309,14 +455,14 @@ void ScrDrvSetCrs( int x, int y )
     screen.crsY=y;
     screen.crsX=x ;
     if( screen.crsActiv )
-	WKBiosSetCursorPos( 0, screen.crsY, screen.crsX ) ;
+	WKBiosSetCursorPos( VIDEOPAGE, screen.crsY, screen.crsX ) ;
 }
 
 
 void ScrDrvHideCrs()
 {
     screen.crsActiv = 0 ;
-    WKBiosSetCursorPos( 0, screen.sy, 0) ;  /* will hide cursor */
+    WKBiosSetCursorPos( VIDEOPAGE, screen.sy, 0) ;  /* will hide cursor */
 }
 
 void ScrDrvMouseOn()
@@ -343,6 +489,8 @@ void ScrDrvShowCrs( int form )
     else if( form == 1 ) {
       #if OS2
 	WKBiosSetCursorForm( 0x0c0f ) ;
+      #elif WINNT
+	/*WKBiosSetCursorForm( 0x0c0f )*/ ;
       #else
 	WKBiosSetCursorForm( isColor? 0x0607:  0x0b0c ) ;
       #endif
@@ -350,13 +498,15 @@ void ScrDrvShowCrs( int form )
     } else if( form == 2 ) {
       #if OS2
 	WKBiosSetCursorForm( 0x070f ) ;
+      #elif WINNT
+	/*WKBiosSetCursorForm( 0x070f )*/ ;
       #else
 	WKBiosSetCursorForm( isColor? 0x0307:0x060c ) ;
       #endif
 	screen.crsForm = 2 ;
     }
     screen.crsActiv = 1 ;
-    WKBiosSetCursorPos( 0, screen.crsY, screen.crsX ) ;
+    WKBiosSetCursorPos( VIDEOPAGE, screen.crsY, screen.crsX ) ;
 }
 
 
@@ -383,6 +533,7 @@ void ScrDrvSaveScreen()
     cnt  = save.orgScrSize;
     while( cnt-- )
 	*(mPtr++) = *(sPtr++);
+  #elif WINNT
   #else
     sPtr = screenBase;
     cnt  = save.orgScrSize;
@@ -419,6 +570,7 @@ void ScrDrvRestScreen()
     cnt  = save.orgScrSize;
     while( cnt-- )
 	*(sPtr++) = *(mPtr++);
+  #elif WINNT
   #else
     sPtr = screenBase;
     cnt  = save.orgScrSize;
@@ -429,12 +581,13 @@ void ScrDrvRestScreen()
     MouseShow();
   #endif
     WKBiosSetCursorForm( save.cursorForm ) ;
-    WKBiosSetCursorPos( 0, save.crsY, save.crsX ) ;
+    WKBiosSetCursorPos( VIDEOPAGE, save.crsY, save.crsX ) ;
 }
 
 
 
-void ScrDrvShow()
+void
+ScrDrvShow()
 {
   #if OS20
     MouseHide();
@@ -444,16 +597,41 @@ void ScrDrvShow()
     MouseHide();
     VioShowBuf( 0, screen.size*2, hvio);
     MouseShow();
+  #elif WINNT
+    if( vio_changed ) {
+	COORD i_size, i_pos;
+	SMALL_RECT rect;
+
+	i_size.X = screen.sx;
+	i_size.Y = screen.sy;
+	i_pos.X = viorect.Left;
+	i_pos.Y = viorect.Top;
+	if( !WriteConsoleOutput(con_out, image, i_size, i_pos, &viorect ) )
+	    Error(0,"WriteConsoleOutput failed: rc=%d", GetLastError());
+	vio_changed = 0;
+	viorect.Left  = screen.sx;
+	viorect.Right = 0;
+	viorect.Top = screen.sy;
+	viorect.Bottom = 0;
+    }
   #endif
 }
 
 
-void ScrDrvRepaint()
+void
+ScrDrvRepaint()
 {
     int i;
 
     if( crtSaveFlag )
 	ScrDrvPauseCrt(0);
+#ifdef WINNT
+  vio_changed = 0;
+  viorect.Left	= screen.sx;
+  viorect.Right = 0;
+  viorect.Top = screen.sy;
+  viorect.Bottom = 0;
+#else
   #if OS2
     if( !vioBuffer )
 	return;
@@ -472,6 +650,7 @@ void ScrDrvRepaint()
   #if !OS2
     MouseShow();
   #endif
+#endif
 }
 
 
@@ -481,26 +660,35 @@ void ScrDrvRepaint()
  * Returns: Number of Buttons or 0 if no Mouse available in this screen Area
  */
 
-int ScrDrvQryMouse( int *btn, int *px, int *py )
+int
+ScrDrvQryMouse( int *btn, int *px, int *py )
 {
     *btn = MouseGet( px, py );
     return screen.buttons;
 }
 
 
-void ScrDrvReadCell( int x, int y, int *c, int *a )
+void
+ScrDrvReadCell( int x, int y, int *c, int *a )
 {
     int i;
 
     i = y * screen.sx + x ;
     if( i >= 0 && i < screen.size ) {
+      #ifdef WINNT
+	*c = translateCodes ? MapIbm2Iso( image[i].Char.AsciiChar, 1 )
+			    : image[i].Char.AsciiChar;
+	*a = image[i].Attributes;
+      #else
 	*c = translateCodes ? MapIbm2Iso( image[i].c, 1 ) : image[i].c;
 	*a = image[i].a;
+      #endif
     }
 }
 
 
-void ScrDrvWriteCell( int x, int y, int c, int a )
+void
+ScrDrvWriteCell( int x, int y, int c, int a )
 {
     int i;
 
@@ -515,18 +703,27 @@ void ScrDrvWriteCell( int x, int y, int c, int a )
 	if( translateCodes )
 	    c = MapIso2Ibm(c,1);
     if( i >= 0 && i < screen.size ) {
+      #ifdef WINNT
+	if( image[i].Char.AsciiChar != c || image[i].Attributes != a ) {
+	    image[i].Char.AsciiChar = c ;
+	    image[i].Attributes = a ;
+	    vio_changed = 1;
+	    if( x < viorect.Left )
+		viorect.Left = x;
+	    if( x > viorect.Right )
+		viorect.Right = x;
+	    if( y < viorect.Top )
+		viorect.Top = y;
+	    if( y > viorect.Bottom )
+		viorect.Bottom = y;
+	}
+      #else
 	if( image[i].c != c || image[i].a != a ) {
 	    image[i].c = c ;
 	    image[i].a = a ;
 	  #if !OS2
 	    MouseHideCond( x, y, x+1, y);
 	  #endif
-	    #if __ZTC__ && DEBUG && !OS2 /* Tell ZDB to switch to output Screen */
-	    {
-	      int dmy1, dmy2;	  /* by calling video Bios */
-	      WKBiosGetVideoMode( &dmy1, &dmy2 );
-	    }
-	    #endif
 	  #if OS2
 	    vioBuffer[i] = image[i];
 	  #else /* DOS*/
@@ -536,6 +733,7 @@ void ScrDrvWriteCell( int x, int y, int c, int a )
 	    MouseShow();
 	  #endif
 	}
+      #endif
     }
 }
 
@@ -559,6 +757,22 @@ ScrDrvWriteStr( int x, int y, const char *buf, size_t len, int a )
 	    if( c & 0x80 )
 		if( translateCodes )
 		    c = MapIso2Ibm(c,1);
+	  #ifdef WINNT
+	   if( image[i].Char.AsciiChar != c || image[i].Attributes != a ) {
+	       image[i].Char.AsciiChar = c ;
+	       image[i].Attributes = a ;
+	       vio_changed = 1;
+	       if( x < viorect.Left )
+		   viorect.Left = x;
+	       if( x > viorect.Right )
+		   viorect.Right = x;
+	       if( y < viorect.Top )
+		   viorect.Top = y;
+	       if( y > viorect.Bottom )
+		   viorect.Bottom = y;
+	    }
+	    if( ++x > screen.x2 ) { x=0; y++; }
+	  #else
 	    if( image[i].c != c || image[i].a != a ) {
 		image[i].c = c ;
 		image[i].a = a ;
@@ -567,12 +781,6 @@ ScrDrvWriteStr( int x, int y, const char *buf, size_t len, int a )
 		  #if !OS2
 		    MouseHideCond( x, y, x+len, y);
 		  #endif
-		    #if __ZTC__ && DEBUG && !OS2 /* Tell ZDB */
-		    {
-		      int dmy1, dmy2;	  /* by calling video Bios */
-		      WKBiosGetVideoMode( &dmy1, &dmy2 );
-		    }
-		    #endif
 		}
 	      #if OS2 && 0
 		VioWrtCellStr( (PCH)&image[i], 2, i/screen.sx, i%screen.sx,hvio);
@@ -582,8 +790,9 @@ ScrDrvWriteStr( int x, int y, const char *buf, size_t len, int a )
 		screenBase[i] = image[i];
 	      #endif
 	    }
+	  #endif
 	}
-  #if !OS2
+  #if !OS2 && !WINNT
     if( anyupd )
 	MouseShow();
   #endif
@@ -611,6 +820,22 @@ ScrDrvWriteAttrStr( int x, int y, const char *buf,
 	    if( c & 0x80 )
 		if( translateCodes )
 		    c = MapIso2Ibm(c,1);
+	  #ifdef WINNT
+	   if( image[i].Char.AsciiChar != c || image[i].Attributes != a ) {
+	       image[i].Char.AsciiChar = c ;
+	       image[i].Attributes = a ;
+	       vio_changed = 1;
+	       if( x < viorect.Left )
+		   viorect.Left = x;
+	       if( x > viorect.Right )
+		   viorect.Right = x;
+	       if( y < viorect.Top )
+		   viorect.Top = y;
+	       if( y > viorect.Bottom )
+		   viorect.Bottom = y;
+	    }
+	    if( ++x > screen.x2 ) { x=0; y++; }
+	  #else
 	    if( image[i].c != c || image[i].a != a ) {
 		image[i].c = c ;
 		image[i].a = a ;
@@ -619,12 +844,6 @@ ScrDrvWriteAttrStr( int x, int y, const char *buf,
 		  #if !OS2
 		    MouseHideCond( x, y, x+len, y);
 		  #endif
-		    #if __ZTC__ && DEBUG && !OS2 /* Tell ZDB */
-		    {
-		      int dmy1, dmy2;	  /* by calling video Bios */
-		      WKBiosGetVideoMode( &dmy1, &dmy2 );
-		    }
-		    #endif
 		}
 	      #if OS2 && 0
 		VioWrtCellStr( (PCH)&image[i], 2, i/screen.sx, i%screen.sx,hvio);
@@ -634,8 +853,9 @@ ScrDrvWriteAttrStr( int x, int y, const char *buf,
 		screenBase[i] = image[i];
 	      #endif
 	    }
+	  #endif
 	}
-  #if !OS2
+  #if !OS2 && !WINNT
     if( anyupd )
 	MouseShow();
   #endif
@@ -667,6 +887,21 @@ void ScrDrvWriteStrFill( int x, int y, const char *buf,
 	    if( c & 0x80 )
 		if( translateCodes )
 		    c = MapIso2Ibm(c,1);
+	  #ifdef WINNT
+	    if( image[i].Char.AsciiChar != c  ) {
+	       image[i].Char.AsciiChar = c ;
+	       vio_changed = 1;
+	       if( x < viorect.Left )
+		   viorect.Left = x;
+	       if( x > viorect.Right )
+		   viorect.Right = x;
+	       if( y < viorect.Top )
+		   viorect.Top = y;
+	       if( y > viorect.Bottom )
+		   viorect.Bottom = y;
+	    }
+	    if( ++x > screen.x2 ) { x=0; y++; }
+	  #else
 	    if( image[i].c != c ) {
 		image[i].c = c ;
 		if( !anyupd ) {
@@ -674,12 +909,6 @@ void ScrDrvWriteStrFill( int x, int y, const char *buf,
 		  #if !OS2
 		    MouseHideCond( x, y, x+len, y);
 		  #endif
-		    #if __ZTC__ && DEBUG && !OS2 /* Tell ZDB */
-		    {
-		      int dmy1, dmy2;	  /* by calling video Bios */
-		      WKBiosGetVideoMode( &dmy1, &dmy2 );
-		    }
-		    #endif
 		}
 	      #if OS2 && 0
 		VioWrtCellStr( (PCH)&image[i], 2, i/screen.sx, i%screen.sx,hvio);
@@ -689,8 +918,24 @@ void ScrDrvWriteStrFill( int x, int y, const char *buf,
 		screenBase[i] = image[i];
 	      #endif
 	    }
+	  #endif
 	}
 	for( ; maxlen && i < screen.size; maxlen--,i++ ) {
+	  #ifdef WINNT
+	    if( image[i].Char.AsciiChar != ' ' ) {
+	       image[i].Char.AsciiChar = ' ';
+	       vio_changed = 1;
+	       if( x < viorect.Left )
+		   viorect.Left = x;
+	       if( x > viorect.Right )
+		   viorect.Right = x;
+	       if( y < viorect.Top )
+		   viorect.Top = y;
+	       if( y > viorect.Bottom )
+		   viorect.Bottom = y;
+	    }
+	    if( ++x > screen.x2 ) { x=0; y++; }
+	  #else
 	    if( image[i].c != ' ' ) {
 		image[i].c = ' ' ;
 		if( !anyupd ) {
@@ -698,12 +943,6 @@ void ScrDrvWriteStrFill( int x, int y, const char *buf,
 		  #if !OS2
 		    MouseHideCond( x, y, x+len, y);
 		  #endif
-		    #if __ZTC__ && DEBUG && !OS2 /* Tell ZDB */
-		    {
-		      int dmy1, dmy2;	  /* by calling video Bios */
-		      WKBiosGetVideoMode( &dmy1, &dmy2 );
-		    }
-		    #endif
 		}
 	      #if OS2 && 0
 		VioWrtCellStr( (PCH)&image[i], 2, i/screen.sx, i%screen.sx,hvio);
@@ -713,9 +952,10 @@ void ScrDrvWriteStrFill( int x, int y, const char *buf,
 		screenBase[i] = image[i];
 	      #endif
 	    }
+	  #endif
 	}
     }
-  #if !OS2
+  #if !OS2 && !WINNT
     if( anyupd )
 	MouseShow();
   #endif
@@ -740,6 +980,22 @@ void ScrDrvWriteNChar( int x, int y, int c, int a, size_t n )
 	    c = MapIso2Ibm(c,1);
     if( i >= 0 )
 	for( ; n && i < screen.size; n--, i++ ) {
+	  #ifdef WINNT
+	    if( image[i].Char.AsciiChar != c || image[i].Attributes != a ) {
+	       image[i].Char.AsciiChar = c ;
+	       image[i].Attributes = a ;
+	       vio_changed = 1;
+	       if( x < viorect.Left )
+		   viorect.Left = x;
+	       if( x > viorect.Right )
+		   viorect.Right = x;
+	       if( y < viorect.Top )
+		   viorect.Top = y;
+	       if( y > viorect.Bottom )
+		   viorect.Bottom = y;
+	    }
+	    if( ++x > screen.x2 ) { x=0; y++; }
+	  #else
 	    if( image[i].c != c || image[i].a != a ) {
 		image[i].c = c ;
 		image[i].a = a ;
@@ -748,12 +1004,6 @@ void ScrDrvWriteNChar( int x, int y, int c, int a, size_t n )
 		  #if !OS2
 		    MouseHideCond( x, y, x+n, y);
 		  #endif
-		    #if __ZTC__ && DEBUG && !OS2 /* Tell ZDB */
-		    {
-		      int dmy1, dmy2;	  /* by calling video Bios */
-		      WKBiosGetVideoMode( &dmy1, &dmy2 );
-		    }
-		    #endif
 		}
 	      #if OS2 && 0
 		VioWrtCellStr( (PCH)&image[i], 2, i/screen.sx, i%screen.sx,hvio) ;
@@ -763,8 +1013,9 @@ void ScrDrvWriteNChar( int x, int y, int c, int a, size_t n )
 		screenBase[i] = image[i];
 	      #endif
 	    }
+	  #endif
 	}
-  #if !OS2
+  #if !OS2 && !WINNT
     if( anyupd )
 	MouseShow();
   #endif
@@ -789,6 +1040,21 @@ void ScrDrvWriteNAttr( int x, int y, int a, size_t n )
     i = y * screen.sx + x ;
     if( i >= 0 )
 	for( ; n && i < screen.size; n--, i++ ) {
+	  #ifdef WINNT
+	    if( image[i].Attributes != a ) {
+	       image[i].Attributes = a ;
+	       vio_changed = 1;
+	       if( x < viorect.Left )
+		   viorect.Left = x;
+	       if( x > viorect.Right )
+		   viorect.Right = x;
+	       if( y < viorect.Top )
+		   viorect.Top = y;
+	       if( y > viorect.Bottom )
+		   viorect.Bottom = y;
+	    }
+	    if( ++x > screen.x2 ) { x=0; y++; }
+	  #else
 	    if( image[i].a != a ) {
 		image[i].a = a ;
 		if( !anyupd ) {
@@ -796,12 +1062,6 @@ void ScrDrvWriteNAttr( int x, int y, int a, size_t n )
 		  #if !OS2
 		    MouseHideCond( x, y, x+n, y);
 		  #endif
-		    #if __ZTC__ && DEBUG && !OS2 /* Tell ZDB */
-		    {
-		      int dmy1, dmy2;	  /* by calling video Bios */
-		      WKBiosGetVideoMode( &dmy1, &dmy2 );
-		    }
-		    #endif
 		}
 	      #if OS2 && 0
 		VioWrtCellStr( (PCH)&image[i], 2, i/screen.sx, i%screen.sx,hvio) ;
@@ -811,8 +1071,9 @@ void ScrDrvWriteNAttr( int x, int y, int a, size_t n )
 		screenBase[i] = image[i];
 	      #endif
 	    }
+	  #endif
 	}
-  #if !OS2
+  #if !OS2 && !WINNT
     if( anyupd )
 	MouseShow();
   #endif
@@ -843,13 +1104,18 @@ void ScrDrvPauseCrt( int mode )
 	return;
   #endif
     if( mode ) {
-      #if __ZTC__ && DEBUG && !OS2  /* Tell ZDB to switch to output Screen */
+      #if __ZTC__ && DEBUG && !OS2 && !WINNT  /* Tell ZDB to switch to output Screen */
 	int dmy1, dmy2;     /* by calling video Bios */
 	WKBiosGetVideoMode( &dmy1, &dmy2 );
       #endif
+      #ifdef WINNT
+	cell.Char.AsciiChar = ' ';
+	cell.Attributes = 7;
+      #else
 	cell.c = ' ';
 	cell.a = 7;
-      #if !OS2
+      #endif
+      #if !OS2 && !WINNT
 	MouseHide();
       #endif
 	cnt  = save.orgScrSize;
@@ -873,6 +1139,8 @@ void ScrDrvPauseCrt( int mode )
 	}
 	for(i=0; i < 3; i++ ) /* write some random cells */
 	    sPtr[ rand() % cnt ] = GetRndCell();
+      #elif WINNT
+	/* FIXME */;
       #else
 	sPtr = screenBase;
 	if( !crtSaveFlag ) {
@@ -888,7 +1156,7 @@ void ScrDrvPauseCrt( int mode )
     else if( crtSaveFlag ) {
 	crtSaveFlag = 0;
 	ScrDrvRepaint();
-  #if !OS2
+  #if !OS2 && !WINNT
 	MouseShow();
   #endif
     }
@@ -902,33 +1170,39 @@ void ScrDrvPauseCrt( int mode )
  */
 
 
-static cell_t GetRndCell()
+static cell_t
+GetRndCell()
 {
     cell_t val;
-
+#ifdef WINNT
+  #define X val.Char.AsciiChar
+#else
+  #define X val.c
+#endif
     switch( rand() % 1000 ) {
-      case 1: val.c = '.';        break;
-      case 2: val.c = '.';        break;
-      case 3: val.c = '.';        break;
-      case 4: val.c = '.';        break;
-      case 5: val.c = '.';        break;
-      case 6: val.c = '.';        break;
-      case 7: val.c = '.';        break;
-      case 8: val.c = '.';        break;
-      case 9: val.c = '+';        break;
-      case 10:val.c = '+';       break;
-      case 11:val.c = '+';       break;
-      case 12:val.c = '*';       break;
-      case 13:val.c = '.';       break;
-      case 14:val.c = 'w';       break;
-      case 15:val.c = '.';       break;
-      case 16:val.c = '*';       break;
-      case 17:val.c = 'W';       break;
-      case 18:val.c = '.';       break;
-      default:val.c = ' ';        break;
+      case 1:  X = '.';        break;
+      case 2:  X = '.';        break;
+      case 3:  X = '.';        break;
+      case 4:  X = '.';        break;
+      case 5:  X = '.';        break;
+      case 6:  X = '.';        break;
+      case 7:  X = '.';        break;
+      case 8:  X = '.';        break;
+      case 9:  X = '+';        break;
+      case 10: X = '+';       break;
+      case 11: X = '+';       break;
+      case 12: X = '*';       break;
+      case 13: X = '.';       break;
+      case 14: X = 'w';       break;
+      case 15: X = '.';       break;
+      case 16: X = '*';       break;
+      case 17: X = 'W';       break;
+      case 18: X = '.';       break;
+      default: X = ' ';        break;
     }
-    val.a = (rand() % 6)+1;
+    X = (rand() % 6)+1;
     return val;
+#undef X
 }
 
 
@@ -943,7 +1217,7 @@ unsigned ScrDrvGetToggleKeys()
 {
     unsigned state;
 
-  #if OS20
+  #if OS20 || WINNT
     state = WKKeyboardGetToggle();
   #elif OS2
     KBDINFO kbi;
@@ -979,14 +1253,14 @@ ScrDrvBasicInkey( ushort *rVal, ushort *rScan, unsigned *rFlags )
     int c, state, i, msX, msY;
     unsigned msBtn;
   static unsigned lastMsBtn;
-  #if OS20
+  #if OS20 || WINNT
     unsigned kstate;
   #elif OS2
     KBDKEYINFO kbci;
   #endif
 
     chr = scan = flags = 0;
-  #if OS20
+  #if OS20 || WINNT
     if( WKKeyboardGetChar( 0, &chr, &scan, &kstate ) )
 	return 0;/* fehler (z.B. kein focus */
   #elif OS2
@@ -1038,7 +1312,7 @@ ScrDrvBasicInkey( ushort *rVal, ushort *rScan, unsigned *rFlags )
   #endif
 
 
-  #if OS20
+  #if OS20 || WINNT
     flags |= kstate;
   #elif OS2
     if( kbci.fsState & (KBDSTF_RIGHTSHIFT| KBDSTF_LEFTSHIFT) )
@@ -1074,7 +1348,7 @@ ScrDrvBasicInkey( ushort *rVal, ushort *rScan, unsigned *rFlags )
 
 void ScrDrvPoll(void)
 {
-#if !OS2
+#if !OS2 && !WINNT
     int state, c;
   #if DOS386 && __ZTC__
     static short _far *fp;
