@@ -723,7 +723,7 @@ static void UpdBrowse(void)
     sx = sxBrow;
     sy = syBrow;
 
-    GetLineBlock( browseFileHd, crsY, text, lenTbl,
+    GetLineBlock( browseFileHd, filY, text, lenTbl,
 			  sy+1 , &ml1, &mp1, &ml2, &mp2);
     if( monochromePalette )
 	sy--;
