@@ -216,7 +216,7 @@ int Cmd_Assign( cmd_t *cmd )
 	strcat( buf, "=" );
 	strcat( buf, val );
 	free( val );
-	if( putenv( buf ) )
+	if( myputenv( buf ) )
 	    err = ERR_PUTENV;
 	free( buf );
     }
