@@ -82,6 +82,7 @@ static APP_PARAM app_options[] = {
   { "-d",           &prm.display_name   },
   { "-geometry",    &prm.geometry_cline },
   { "-g",           &prm.geometry_cline },
+  { "-fn",          &prm.font           },
   { NULL }
 };
 
@@ -310,6 +311,19 @@ JnxInitApp( int x, int y, unsigned width, unsigned height )
     else
 	Error(0,"error creating icon");
 
+  #warning Must implemt hits for wmaker.
+#if 0
+    xwmh.window_group = ...
+
+
+You must define the WM_CLASS (XSetClassHint()) and the
+CLIENT_LEADER or XWMHints.window_group properties, which are
+automatically set by most applications that use Xt (Motif, Athena ...), but if
+you use plain Xlib you must set them by hand.
+
+Also you must make a call to XSetCommand(dpy, leader, argv, argc);
+#endif
+
     XSetWMHints(glo.display, glo.mainWin, &xwmh );
 
     /* set attributes */
@@ -546,7 +560,7 @@ WaitEvent()
 
 
 /****************
- * handle the buttonPeree (pressed=1) and ButtonRelease (pressed = 0) events.
+ * handle the buttonPress (pressed=1) and ButtonRelease (pressed = 0) events.
  */
 static void
 ProcessButtonEvent(XButtonEvent *e,  int pressed )
@@ -658,7 +672,9 @@ JnxReadKeyCode( unsigned *keycode, unsigned *state )
 	if( glo.btn[idx].last.state & Mod1Mask )
 	    n |= (1<<2);
 	if( glo.btn[idx].last.state & Mod1Mask )
-	    n |= (1<<3); /* what is AltGr? */
+	    n |= (1<<3); /* This is AltGr on my old XFree86 */
+	if( glo.btn[idx].last.state & 0x2000 )
+	    n |= (1<<3); /* This is AltGr on my XFree86 3.3 with the pc102 */
 	*state = n;
 	return 1;
     }
@@ -692,6 +708,11 @@ JnxReadKeyCode( unsigned *keycode, unsigned *state )
 		n = 98; break;
 	/* 113:   Mode_switch*/
 	/* 114:   Break*/
+      case 115: /* Super_L */
+      case 116: /* Super_R */
+      case 117: /* Hyper_R */
+	break;
+
       default:
 	return 0;   /* throw away */
     }
@@ -704,7 +725,9 @@ JnxReadKeyCode( unsigned *keycode, unsigned *state )
     if( lastkey.state & Mod1Mask )
 	n |= (1<<2);
     if( lastkey.state & Mod1Mask )
-	n |= (1<<3); /* what is AltGr? */
+	n |= (1<<3); /* This is AltGr on my old XFree86 */
+    if( lastkey.state & 0x2000 )
+	n |= (1<<3); /* This is AltGr on my XFree86 3.3 with the pc102 */
     *state   = n;
     return 1;
 }
@@ -857,7 +880,14 @@ JnxWriteChar( int x, int y, int c )
     else if( y1 != y || x1 + n != x )
 	goto flush; /* ;-)) */
 
-    buf[n++] = c;
+    /* we use iso8879-1 for output.
+     * all control characters are filtered away and replaced by a centered dot;
+     * otherwise the Xserver garbles the output.
+     */
+    if( !(c & 0x60) || c == 127 )
+	buf[n++] = 0xb7;
+    else
+	buf[n++] = c;
 }
 
 void
