@@ -27,7 +27,7 @@
  */
 
 #include <wk/tailor.h>
-RCSID("$Id: bvideo.c,v 1.11 1996/01/25 20:16:33 wernerk Exp $")
+RCSID("$Id: bvideo.c,v 1.13 1997/10/06 12:54:47 wk Exp $")
 #include <stdio.h>
 #include <stdlib.h>
 #include <wk/bios.h>
@@ -39,6 +39,7 @@ RCSID("$Id: bvideo.c,v 1.11 1996/01/25 20:16:33 wernerk Exp $")
 #elif defined( OS2 ) /* or make WK16WRAP */
     #define INCL_SUB 1
     #include <os2.h>
+#elif WINNT
 #elif UNIX
     /* termios.h? */
 #else
@@ -164,6 +165,8 @@ int FName(BiosGetVideoMode) ( int *cols, int *rows )
     }
     else
 	return 7 ; /* MDA 80*25*/
+#elif WINNT
+    return 0;  /* FIXME */
 #elif defined(UNIX)
     if( rows ) *rows = 25;
     if( cols ) *cols = 80;
@@ -355,6 +358,8 @@ void FName(BiosSetVideoMode)( int modi )
       default:	return; /* can't use BUG here */
     }
     VioSetMode( &viomi, 0 ) ;
+#elif WINNT
+	       /* FIXME */
 #elif defined(UNIX)
 	/* not implemented */
 #else
@@ -402,6 +407,15 @@ void FName(BiosGetCursorPos)( int p, int *y , int *x )
     VioGetCurPos( &row, &col, 0 ) ;
     *y = row ;
     *x = col ;
+#elif WINNT
+    CONSOLE_SCREEN_BUFFER_INFO info;
+
+    if( !GetConsoleScreenBufferInfo( (HANDLE)p, &info ) )
+	*y = *x = 0;
+    else {
+	*x = info.dwCursorPosition.X;
+	*y = info.dwCursorPosition.Y;
+    }
 #elif defined(UNIX)
     /*we should use tc... functions ansd curses or terminfo*/
     *y = 0;
@@ -428,10 +442,11 @@ void FName(BiosGetCursorPos)( int p, int *y , int *x )
  #include <wk/bios.h>
 
  void WKBiosSetCursorPos( p, y, x );
- int	p ;		Page
+ int	p ;		Page or console handle for Win32.
  int	y, x ;		New cursorposition
 @Description
  Die Funktion setzt den Cursor auf die angegebene Position
+
 @See Also
  WKBiosGetCursorPos
 #endif
@@ -448,6 +463,12 @@ void FName(BiosSetCursorPos)( int p, int y , int x )
 #elif defined(OS2)
     VioSetCurPos( y, x , 0 ) ;
     p = p ; /* not used */
+#elif WINNT
+    COORD coord;
+
+    coord.X = x;
+    coord.Y = y;
+    SetConsoleCursorPosition((HANDLE)p,coord);
 #elif defined(UNIX)
     /* to be implmented */
 #else
@@ -497,6 +518,8 @@ unsigned FName(BiosGetCursorForm)()
     if( vioci.attr == -1 )  /* PROBLEM: compares alwas results in false */
 	f |= 0x2000 ;	/* cursor is hidden */
     return f ;
+#elif WINNT
+    return 0;  /* FIXME */
 #elif defined(UNIX)
     return 0;
 #else
@@ -565,6 +588,8 @@ void FName(BiosSetCursorForm)( unsigned f )
     vioci.cx	 = 0 ;
     vioci.attr	 = (f & 0x2000)? -1:0 ;
     VioSetCurType( &vioci, 0 ) ;
+#elif WINNT
+	       /* FIXME */
 #elif defined(UNIX)
     /* ... WORK ... */
 #else
