@@ -38,7 +38,7 @@
  */
 
 #include <wk/tailor.h>
-RCSID("$Id: direc2.c,v 1.23 1996/08/26 17:51:56 wk Exp $")
+RCSID("$Id: direc2.c,v 1.25 1997/05/21 14:46:01 wk Exp $")
 #if EMX
   #include <sys/emx.h>
 #endif
@@ -47,7 +47,11 @@ RCSID("$Id: direc2.c,v 1.23 1996/08/26 17:51:56 wk Exp $")
 #include <string.h>
 #include <errno.h>
 #include <wk/lib.h>	/* to define MSDOS or something else */
-#if (UNIX  || POSIX) && !EMX
+#if WINNT && __WATCOMC__
+    #include <dos.h>
+#elif __MINGW32__
+    #include <dir.h>
+#elif (UNIX  || POSIX) && !EMX && !WINNT
     #include <sys/types.h>
     #include <dirent.h>
     #include <fnmatch.h>    /* is this posix? (seems okay for openVMS) */
@@ -84,7 +88,7 @@ RCSID("$Id: direc2.c,v 1.23 1996/08/26 17:51:56 wk Exp $")
 #include <wk/string.h>
 
 #if !__WATCOMC__ && !__MSC__ && !__ZTC__ && !EMX && \
-    !OS2 && !UNIX && !POSIX
+    !OS2 && !UNIX && !POSIX && !WINNT
    #error unsupported Compiler/OS
 #endif
 
@@ -97,7 +101,13 @@ RCSID("$Id: direc2.c,v 1.23 1996/08/26 17:51:56 wk Exp $")
 /****** types ********/
 typedef struct {
 	int inuse;
-      #if (UNIX  || NETWARE  || POSIX) && !EMX
+      #if WINNT && __WATCOMC__
+	struct find_t fi;
+      #elif WINNT && !__CYGWIN32__
+	struct _finddata_t fi;
+	int hdir;
+	int hdirActive;
+      #elif (UNIX  || NETWARE  || POSIX) && !EMX
 	DIR *dirp;
 	struct dirent *ent;/* may point to data local to dirp, see readdir()*/
 	struct stat statbuf;
@@ -150,7 +160,12 @@ static void CleanUp( void *dummy )
 		if( handleTbl[i].hdirActive ) {
 		    DosFindClose(handleTbl[i].hdir) ;
 		    handleTbl[i].hdirActive = 0;
-		};
+		}
+	      #elif WINNT
+		if( handleTbl[i].hdirActive ) {
+		    _findclose(handleTbl[i].hdir);
+		    handleTbl[i].hdirActive = 0;
+		}
 	      #endif
 		handleTbl[i].inuse = 0 ;
 	    }
@@ -196,7 +211,7 @@ static void CleanUp( void *dummy )
 
 int FindFile( int mode, char *buffer, size_t bufSize,
 	      unsigned *flags, int *handle )
-#if (UNIX || NETWARE || POSIX) && !EMX
+#if (UNIX || NETWARE || POSIX) && !EMX	&& !WINNT
 {
     int i, err ;
     unsigned attr ;
@@ -376,6 +391,9 @@ int FindFile( int mode, char *buffer, size_t bufSize,
     #if EMX && !OS20
       struct _find *efi ;
     #endif
+  #elif WINNT
+    struct _finddata_t *fi ;
+    int hdir;
   #else
     struct find_t *fi ;
   #endif
@@ -414,6 +432,8 @@ int FindFile( int mode, char *buffer, size_t bufSize,
 		handleTbl[i].inuse = 1 ;
 	      #if OS2 || EMX
 		handleTbl[i].hdirActive = 0;
+	      #elif WINNT
+		handleTbl[i].hdirActive = 0;
 	      #endif
 		*handle = i ;
 		break ;
@@ -433,6 +453,8 @@ int FindFile( int mode, char *buffer, size_t bufSize,
 	#if EMX && !OS20
 	  efi = &handleTbl[*handle].efi;
 	#endif
+      #elif WINNT
+	hdir =		handleTbl[*handle].hdir;
       #endif
 	entry = handleTbl + *handle;
     }
@@ -498,6 +520,16 @@ int FindFile( int mode, char *buffer, size_t bufSize,
        #if EMX && !OS20
 	}
        #endif
+      #elif WINNT
+	hdir = _findfirst( buffer, fi );
+	if( hdir == -1 ) {
+	    err = -1;
+	    errno = ENOENT;
+	}
+	else {
+	    handleTbl[*handle].hdirActive=1;
+	    handleTbl[*handle].hdir = hdir;
+	}
       #else
        #if __ZTC__
 	if( err = _dos_findfirst( buffer, attr, fi ) )
@@ -528,6 +560,10 @@ int FindFile( int mode, char *buffer, size_t bufSize,
 	    errno = ENOENT;
 	else
 	    handleTbl[*handle].usSearchCount = usSearchCount;
+      #elif WINNT
+	err = _findnext( hdir, fi );
+	if( err == -1 )
+	    errno = ENOENT;
       #else
        #if __ZTC__
 	if( err = _dos_findnext( fi ) )
@@ -624,6 +660,11 @@ int FindFile( int mode, char *buffer, size_t bufSize,
 		DosFindClose(handleTbl[*handle].hdir) ;
 		handleTbl[*handle].hdirActive = 0;
 	    }
+	  #elif WINNT
+	    if( handleTbl[*handle].hdirActive ) {
+		_findclose(handleTbl[*handle].hdir) ;
+		handleTbl[*handle].hdirActive = 0;
+	    }
 	  #endif
 	    handleTbl[*handle].inuse = 0 ;
 	}
@@ -655,7 +696,7 @@ int FindFile( int mode, char *buffer, size_t bufSize,
 
 const char *
 FindFileStat( int *handle, findFileStat_t *buf )
-#if (UNIX || NETWARE || POSIX) && !EMX
+#if (UNIX || NETWARE || POSIX) && !EMX && !WINNT
 {
     if( buf )
 	*buf = handleTbl[*handle].statbuf;
@@ -674,6 +715,8 @@ FindFileStat( int *handle, findFileStat_t *buf )
       struct _find *efi ;
     #endif
     struct tm tm;
+  #elif WINNT
+    struct _finddata_t *fi ;
   #else
     ushort t, d;
     struct find_t *fi ;
@@ -730,6 +773,11 @@ FindFileStat( int *handle, findFileStat_t *buf )
        #if EMX && !OS20
 	}
        #endif
+      #elif WINNT
+	buf->st_size  = fi->size;
+	buf->st_atime = fi->time_access;
+	buf->st_mtime = fi->time_write;
+	buf->st_ctime = fi->time_create;
       #else
 	buf->st_size  = fi->size;
 	d = fi->wr_date;
