@@ -46,8 +46,9 @@
 #elif DOS386
 #elif DOS16RM
 #elif EMX
+#elif WINNT
 #else
-    #error Operating system must either be MSDOS, OS2, or EMX
+    #error Operating system must either be MSDOS, OS2, WINNT or EMX
 #endif
 #include <wk/mouse.h>
 #include <wk/bios.h>
@@ -67,6 +68,7 @@
     #include <os2.h>
     #define INCL_WK_WSCRDRV_GUI 1
     /* #define HAVE_OLD_OS2_HEADERS 1 */
+#elif WINNT
 #else
     #include <bios.h>
     #include <dos.h>
@@ -257,19 +259,17 @@ int ScrDrvOpen()
 
 
 
-
-HAB ScrDrvGetActiveHAB(void)
+#ifdef OS2
+HAB
+#else
+int
+#endif
+ScrDrvGetActiveHAB(void)
 {
     return 0;
 }
 
 
-
-
-
-
-
-
 /****************
  *  enables or disables code translation
  */
